////////////////////////////////////////////////////////////////////////
/*! 
 *  \file   TiffIO_Qt4.cpp
 *  \brief  Plugin to enable TIFF I/O in Qt 3.x and 4.x.
 *  \author JD Gascuel
 * 
 * Copyright JD Gascuel, 2005.
 * 
 * Jean-Dominique.Gascuel@imag.fr
 *
 * This software is a computer program whose purpose is to add TIFF image
 * read/write capabilities to the Qt library, hence to any Qt application.
 * 
 * This software is governed by the CeCILL  license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 *
 * TiffIO is based on the libtiff 3.7.2, with the following copyright :
 *      Copyright (c) 1988-1997 Sam Leffler
 *      Copyright (c) 1991-1997 Silicon Graphics, Inc.
 * See the libtiff-3.7.2/ subdirectory.
 */
////////////////////////////////////////////////////////////////////////
// File History:
//
// $Id: TiffIO_Qt4.cpp,v 1.1 2005/09/29 17:39:46 gascuel Exp $
//
// 2005-09-24 : Creation for Qt 4.0 port.

#include <qglobal.h>
#if QT_VERSION < 0x040000
#error This file should compile only with Qt 4.x or upper. There is an error in the .pri/qmake processing.
#endif

#include <QImage>
#include <QImageIOHandler>
#include <QImageIOPlugin>

extern void commonReadTIFF (QImageIOHandler* handler, QImage& img, int& status);
extern void commonWriteTIFF(QImageIOHandler* handler, const QImage& img, int& status);

class QtTiffImageIOHandler : public QImageIOHandler
{
    int _status;                        //! Remember last read/write success or failure.
public:

    QtTiffImageIOHandler(QIODevice *device)
    {
        setDevice(device);
    }

    virtual QByteArray name() const
    {
        return "tiff";
    }

    virtual bool supportsOption (ImageOption option ) const
    {
        if(option == Endianness) return true;
        return false;
    }

    virtual bool canRead() const
    {
        if( ! device()->isReadable() 
         || format() != "tiff"
        )
            return false;

        char head[4];
        quint64 start = device()->pos();      // Store current pointer position.
        device()->read(head, 4);              // Read first four bytes.
        device()->seek(start);                // Seek back to begin of file.
 
        return head[0]=='M' && head[1]=='M' &&  head[2]==0x00 && (head[3]==0x2a || head[3]==0x2b)
           ||  head[0]=='I' && head[1]=='I' && (head[2]==0x2a || head[2]==0x2b) && head[3]==0x00 ;
    }

    virtual bool read(QImage* img)
    {
        commonReadTIFF(this, *img, _status);
        return _status == 0;
    }

    virtual bool write(const QImage& img)
    {
        commonWriteTIFF(this, img, _status);
        return _status == 0;
    }
};

class TIFFIOPlugin : public QImageIOPlugin
{
public:
    virtual QStringList keys() const
    {
        QStringList list;
            list += "tif";
            list += "tiff";
        return  list;
    }

    virtual Capabilities capabilities(QIODevice *device, const QByteArray &format) const
    {
        int cap = 0;

        if( ! device )
            qWarning("TIFF codec called with null device (file is not readable/writable ?)");

        if( device && device->isReadable() )
        {
            char head[4];
            quint64 start = device->pos();      // Store current pointer position.
            device->read(head, 4);              // Read first four bytes.
            device->seek(start);                // Seek back to begin of file.

            if( head[0]=='M' && head[1]=='M' &&  head[2]==0x00 && (head[3]==0x2a || head[3]==0x2b)
            ||  head[0]=='I' && head[1]=='I' && (head[2]==0x2a || head[2]==0x2b) && head[3]==0x00 
            )
                cap |= QImageIOPlugin::CanRead;
        }

        if( device && device->isWritable() && ( (format.toLower()=="tif") || (format.toLower()=="tiff") ) )
            cap |= QImageIOPlugin::CanWrite;

        return cap;
    }

    virtual QImageIOHandler* create(QIODevice *device, const QByteArray &format = QByteArray()) const
    {
        return new QtTiffImageIOHandler(device);
    }
};

Q_EXPORT_PLUGIN(TIFFIOPlugin);